SystemOrganization addCategory: #'Rest-Tests'!
SystemOrganization addCategory: #'Rest-Tests-Server'!
SystemOrganization addCategory: #'Rest-Tests-Fixtures'!
SystemOrganization addCategory: #'Rest-Tests-Client'!

RestHttpHandler subclass: #RestPerformHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Tests-Fixtures'!

!RestPerformHttpHandler methodsFor: 'request processing' stamp: 'cs 2/19/2009 17:51'!
process: anHTTPRequest
	"Takes first element in path as classname and sends message from third path element to that class."

	|pc class selector result|
	pc := anHTTPRequest uri identifierPath.
	class := pc second asSymbol.
	selector := pc third asSymbol.
	
	Smalltalk at: class ifPresent: [:c | 
		(c class allSelectors includes: selector) ifTrue: [
			result := c perform: selector]].

	^ HTTPResponse ok
		entity: result asString;
		yourself! !

Object subclass: #TestObject
	instanceVariableNames: 'firstname lastname id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Tests-Fixtures'!

!TestObject methodsFor: 'accessing' stamp: 'mfp 2/24/2009 14:22'!
firstname
	^ firstname! !

!TestObject methodsFor: 'accessing' stamp: 'mfp 2/24/2009 14:22'!
firstname: anObject
	firstname := anObject! !

!TestObject methodsFor: 'accessing' stamp: 'mfp 2/24/2009 14:31'!
id
	^ id! !

!TestObject methodsFor: 'accessing' stamp: 'mfp 2/24/2009 14:31'!
id: anObject
	id := anObject! !

!TestObject methodsFor: 'accessing' stamp: 'mfp 2/24/2009 14:23'!
lastname
	^ lastname! !

!TestObject methodsFor: 'accessing' stamp: 'mfp 2/24/2009 14:23'!
lastname: anObject
	lastname := anObject! !

RestResourceHttpHandler subclass: #RestMockResourceHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Tests-Fixtures'!

!RestMockResourceHttpHandler class methodsFor: 'as yet unclassified' stamp: 'mfp 2/24/2009 14:30'!
rootResources
	^ Dictionary newFrom:
		{'tests' -> TestObject }! !

!RestMockResourceHttpHandler class methodsFor: 'as yet unclassified' stamp: 'mfp 2/24/2009 14:31'!
uriMapping

	^ Dictionary newFrom:{
		TestObject -> [:test| '/tests/{1}' format: {test id}]}! !

!Dictionary methodsFor: '*rest-tests' stamp: 'mt 2/16/2009 10:28'!
keysAndValuesCollect: aBlock 
	| newDict |
	newDict := Dictionary new.
	self keysAndValuesDo: [:key :value | newDict add: (aBlock value: key value: value)].
	^ newDict! !

TestCase subclass: #RestSocketTestCase
	instanceVariableNames: 'site'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Tests-Client'!

!RestSocketTestCase class methodsFor: 'accessing' stamp: 'cs 2/19/2009 20:47'!
httpUrl
	^ 'http://www.google.com' asUrl! !

!RestSocketTestCase class methodsFor: 'accessing' stamp: 'topa 2/19/2009 19:10'!
url
	^ 'https://www.verisign.com/favicon.ico' asUrl! !

!RestSocketTestCase methodsFor: 'tests' stamp: 'mig 2/23/2009 16:45'!
testDefaultReturn

	self
		should: ((RestSocket    getDocumentFromUrl: self class httpUrl request: '') isKindOf: RestResponse);
		should: ((RestSocket    getDocumentFromUrl: self class url request: '') isKindOf: RestResponse);
		should: ((RestSocket    putDocumentFromUrl: self class url request: '' content: 'foo') isKindOf: RestResponse);
		should: ((RestSocket   postDocumentFromUrl: self class url request: '' content: 'bar') isKindOf: RestResponse);
		should: ((RestSocket deleteDocumentFromUrl: self class url request: '') isKindOf: RestResponse).! !

TestCase subclass: #RestTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Tests-Server'!

RestTest subclass: #RestHttpServiceTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Tests-Server'!

!RestHttpServiceTest methodsFor: 'running' stamp: 'mt 2/17/2009 16:22'!
setUp
! !

!RestHttpServiceTest methodsFor: 'tests' stamp: 'cs 2/24/2009 18:27'!
testStartAndStop
	
	RestHttpService start.
	(Delay forMilliseconds: 100) wait.
	
	self
		should: (RestHttpService resource isKindOf: RestHttpService);
		should: RestHttpService isRunning.
	
	RestHttpService stop.
	(Delay forMilliseconds: 100) wait.
	
	self
		should: (RestHttpService resource isNil);
		shouldnt: RestHttpService isRunning.! !

RestTest subclass: #RestObjectTest
	instanceVariableNames: 'testProject testUser testTask testObject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Tests-Client'!

!RestObjectTest methodsFor: 'as yet unclassified' stamp: 'mfp 2/25/2009 08:48'!
setUp
	"Install a handler for http service. Add a testObject to the database."

	super setUp.
	RestHttpService resource installHandler: RestMockResourceHttpHandler on: '/'.
	testObject := (TestObject new)
				firstname: 'Test';
				lastname: 'Project';
				id: 1;
				yourself.! !

!RestObjectTest methodsFor: 'as yet unclassified' stamp: 'mfp 2/24/2009 22:37'!
tearDown
	super tearDown.
	testObject := nil.! !

!RestObjectTest methodsFor: 'as yet unclassified' stamp: 'mfp 2/25/2009 08:45'!
testGetRestObject
	| sut uri |
	
	uri := 'tests/'.
		
	sut := RestObject newFromUri: uri.
	
	self 
		should: sut firstname = 'Test';
		should: sut lastname = 'Object';
		should: sut id = 1.! !

RestTest subclass: #RestPerformTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Tests-Server'!

!RestPerformTest methodsFor: 'running' stamp: 'cs 2/24/2009 17:36'!
setUp
	super setUp.
	RestHttpService resource
		installHandler: RestPerformHttpHandler
		on: '/'! !

!RestPerformTest methodsFor: 'tests' stamp: 'mt 2/23/2009 12:18'!
testPort
	| response |
	response := self get: 'RestHttpService/port'.
	self should: response content = RestHttpService port asString.! !

!RestTest class methodsFor: 'testing' stamp: 'cs 2/15/2009 14:14'!
isAbstract
	^ self name = 'RestTest'! !

!RestTest methodsFor: 'helping' stamp: 'mt 2/19/2009 10:11'!
createHttpResponseFrom: aString

	| response contentLength |
	response := HttpResponse fromString: '{"title" : "", "id" : "", "userStories" : {}, "tasks" : {}, "developers" : {}, "explanation" : "", "timeEstimated" : "", "timeActual" : "", "project" : ""}'.
	
"	aString linesDo: [:line|
		(line startsWith: 'HTTP')
			ifTrue: [response status: (HttpResponse statusSymbolFor: (line findTokens: Character space) second asNumber)].
		(line startsWith: 'Content-type')
			ifTrue: [response contentType: (line findTokens: $:) second withBlanksTrimmed].
		(line startsWith: 'Content-length')
			ifTrue: [contentLength := (line findTokens: $:) second asInteger]].
		
	response content: (aString copyFrom: (aString size - contentLength) to: aString size).
"
	^response! !

!RestTest methodsFor: 'helping' stamp: 'mig 2/23/2009 16:45'!
delete: aPath
	"Send an HTTP request with DELETE to localhost at the port of the (singleton) rest HTTP service."
	
	^ RestSocket
		deleteDocumentFromUrl: (('http://localhost:{1}/{2}' format: {RestHttpService port. aPath}) asUrl)
		request: ''! !

!RestTest methodsFor: 'helping' stamp: 'mfp 2/24/2009 14:02'!
get: aPath
	"Send an HTTP request with GET to localhost at the port of the (singleton) rest HTTP service."

	^ RestSocket
			getDocumentFromUrl: ('http://localhost:{1}/{2}' format: {RestHttpService port. aPath}) asUrl
			request: ''! !

!RestTest methodsFor: 'helping' stamp: 'mt 2/18/2009 15:09'!
jsonDataFrom: someFields
	"Create a string for all fields in the dictionary in JSON."
	
	| stream |
	stream := TextStream with: ''.
	(Dictionary newFrom: someFields) jsonWriteOn: stream.	
	^ stream contents! !

!RestTest methodsFor: 'helping' stamp: 'mig 2/23/2009 16:45'!
post: aPath content: aContent
	"Send an HTTP request with POST to localhost at the port of the (singleton) rest HTTP service. Append the given fields to the request body."

	^ RestSocket
		postDocumentFromUrl: (('http://localhost:{1}/{2}' format: {RestHttpService port. aPath}) asUrl)
		request: ''
		content: aContent! !

!RestTest methodsFor: 'helping' stamp: 'mig 2/23/2009 16:45'!
put: aPath content: aContent
	"Send an HTTP request with PUT to localhost at the port of the (singleton) rest HTTP service. Append the given fields to the request body."
	
	^ RestSocket
		putDocumentFromUrl: (('http://localhost:{1}/{2}' format: {RestHttpService port. aPath}) asUrl)
		request: ''
		content: aContent! !

!RestTest methodsFor: 'running' stamp: 'mfp 2/20/2009 10:09'!
setUp
	RestHttpService start.
! !

!RestTest methodsFor: 'running' stamp: 'cs 2/15/2009 14:27'!
tearDown
	RestHttpService stop.! !
